@@ -58,7 +58,7 @@ sub parents {
     @parents;
 }
 
-sub is_type {
+sub is_type($) {
     my ($self, $type) = @_;
     return 1 if $self->name eq $type;
     foreach my $parent ($self->parents) {
@@ -216,8 +216,19 @@ sub relationships {
 sub register_relationship {
     my ($self, $reltype, $params) = @_;
 
+    # scalar: entity
     $params = { entity => $params } unless ref $params;
 
+    # array: name => Entity [, incoming_name ]
+    if (ref $params eq 'ARRAY') {
+
+        $params = {
+            name => $params->[0],
+            entity  => $params->[1],
+            incoming_name => $params->[2],
+        };
+    }
+
     die sprintf("Error: invalid %s relationship for entity '%s': missing 'entity' parameter.", $reltype, $self->name)
         unless $params->{entity};
 
